#include "bullet.h"
#include "level.h"
#include "baby.h"
#include <boost/foreach.hpp>
#include <list>

std::list<Bullet*> Bullet::bulletList;

Bullet::Bullet(float x, float y, int direction) : x(x), y(y), velocity(direction * 7), collided(false) {
	image = ImageMap::get("bullet");
}

void Bullet::checkCollisions() {
	Level* level = Level::current();
	Entity* entity = level->findEntity(x, y, 0);
	
	if (entity == 0) {
		return;
	}

	int type = entity->getType();
	if (type == Tile::BABY) {
		Baby* baby = static_cast<Baby*>(entity);
		baby->cry();
		collided = true;
	}

	if (type == Tile::PLAYER) {
		// can't hit a player
		
		return;
		return;
	}

	if (entity->solid()) {
		collided = true;
	}
}

void Bullet::update() {
	x+= velocity;
	checkCollisions();
}

void Bullet::draw() {
	image->draw(x - 16, y - 16);
}

void Bullet::create(float x, float y, int direction) {
	Bullet* bullet = new Bullet(x, y, direction);
	bulletList.push_back(bullet);
}

bool Bullet::shouldRemove(Bullet* b) {
	return b->collided;
}

void Bullet::updateAll() {
	BOOST_FOREACH(Bullet* b, bulletList) {
		b->update();
	}

	bulletList.remove_if(shouldRemove);
}

void Bullet::drawAll() {
	BOOST_FOREACH(Bullet* b, bulletList) {
		b->draw();
	}
}
