#include "input.h"
#include "image.h"
#include "imagemap.h"
#include "level.h"
#include "leveldata.h"
#include "timer.h"
#include "window.h"
#include "screen.h"
#include "titlescreen.h"
#include "editor.h"
#include "splashscreen.h"
#include "bullet.h"
//#include "audiomixer.h"

#include <iostream>

unsigned int levelID = 0;
Screen* titleScreen;
Screen* aboutScreen;
Screen* splashScreen;
Screen* editor;

Screen* nextScreen = 0;

bool doTransition = false;

int tx;
int dx;

SDL_Surface* transition = 0;

void transitionTo (Screen* s) {
	doTransition = true;
	nextScreen = s;
}

void nextLevel () {

	// Remove all the bullets.
	Bullet::bulletList.clear();

	levelID++;
	
	if (levelID < LevelData::getLevelCount()) {
		nextScreen = LevelData::create(levelID);
	} else {
		nextScreen = titleScreen;
	}
	
	doTransition = true;
}

void reset () {
	nextScreen = LevelData::create(levelID);
}

void startLevel (unsigned int i) {
	levelID = i;
	nextScreen = LevelData::create(levelID);
	
	doTransition = true;
}

void init () {
	
	ImageMap::initVital();
	
	/*splashScreen = new SplashScreen();
	
	Screen::setScreen(splashScreen);
	
	ScreenManager::draw();
	
	SDL_Flip(Window::getSurface());*/
	
	ImageMap::init();
	
	Tile::initialiseTileInfo();
	
	LevelData::init();
	
	// Open menu screen
	titleScreen = new TitleScreen();
	
	editor = new Editor();
	
	Screen::setScreen(titleScreen);
}

void update () {
	if (doTransition) {
		if (transition == 0) {
			bool goBackwards = (nextScreen == titleScreen && Screen::getScreen() != splashScreen);
			tx = goBackwards ? 800 : 0;
			
			dx = goBackwards ? -20 : 20;
			
			SDL_Surface* temp1 = Window::getSurface();
			SDL_Surface* temp2 = SDL_CreateRGBSurface(SDL_SWSURFACE, 800, 640, 32, 0, 0, 0, 0);
			SDL_Surface* temp3 = SDL_CreateRGBSurface(SDL_SWSURFACE, 1600, 640, 32, 0, 0, 0, 0);
			
			Window::setSurface(temp2);
			
			ScreenManager::draw();
			
			SDL_Rect src, dest;

			src.x = 0;
			src.y = 0;
			src.w = 800;
			src.h = 640;
			 
			dest.x = goBackwards ? 800 : 0;
			dest.y = 0;
			dest.w = 800;
			dest.h = 640;
			 
			SDL_BlitSurface(temp2, &src, temp3, &dest);
			
			nextScreen->draw();
			
			dest.x = goBackwards ? 0 : 800;
			SDL_BlitSurface(temp2, &src, temp3, &dest);
		
			transition = SDL_DisplayFormat(temp3);
			
			Window::setSurface(temp1);
			SDL_FreeSurface(temp2);
			SDL_FreeSurface(temp3);
			
		} else {
			tx += dx;
			bool stop = false;
			
			if (dx < 0) {
				stop = (tx < 0);
			} else {
				stop = (tx > 800);
			}
			
			if (stop) {
				SDL_FreeSurface(transition);
				
				doTransition = 0;
				transition = 0;
				
				// recursion is bad, but I am lazy.
				update();
			}
		}
		
		return;
	}
	
	if (nextScreen) {
		Screen::setScreen(nextScreen);
		nextScreen = 0;
	}
	
	ScreenManager::update();
}

void draw () {
	if (transition) {
		SDL_Rect src, dest;

		src.x = tx;
		src.y = 0;
		src.w = 800;
		src.h = 640;
		 
		dest.x = 0;
		dest.y = 0;
		dest.w = 800;
		dest.h = 640;
		 
		SDL_BlitSurface(transition, &src, Window::getSurface(), &dest);
	} else {
		ScreenManager::draw();
	}
	
	SDL_Flip(Window::getSurface());
}

int main(int argc, char *argv[]) {
	if (! Window::init(800, 640, "Crybaby")) {
		exit(1);
	}
	
	/*if (! AudioMixer::init() ) {
		exit(1);
	}*/
	
	Timer timer;
	const double timestep = 1.0/60.0;
	double oldTime = timer.getTime();
	double remainder = 0.0;
	
	init();
	
	while (true) {
		Input::checkEvents();
		
		update();
		draw();
		
		remainder = timer.getTime() - oldTime;

		if (remainder < timestep) {
			SDL_Delay( (int) ((timestep - remainder) * 1000));
		}
		oldTime = timer.getTime();
	}
}

