#ifndef _TITLESCREEN_H_
#define _TITLESCREEN_H_

#include "image.h"
#include "imagemap.h"
#include "main.h"
#include "button.h"
#include "aboutscreen.h"

class TitleScreen : public Screen {
	public:
	
	// Fix segfault by initialising data
	TitleScreen () {
		buttons[0] = new Button(ImageMap::get("play"), 100, 270, playAction);
		buttons[1] = new Button(ImageMap::get("editor"), 100, 350, editorAction);
		buttons[2] = new Button(ImageMap::get("about"), 100, 430, aboutAction);
		buttons[3] = new Button(ImageMap::get("quit"), 100, 510, quitAction);
	}
	
	~TitleScreen () {
		for (int i = 0; i < 4; i++) {
			delete buttons[i];
		}
	}
	
	void handleInput (SDL_Event& event) {
		switch (event.type) {
			case SDL_KEYDOWN:
				keyDown(event);
			break;
			case SDL_QUIT:
				exit(0);
			break;
		}
	}
	
	void keyDown (SDL_Event& event) {
		switch (event.key.keysym.sym) {
			case SDLK_ESCAPE:
				exit(0);
			break;
			
			case SDLK_SPACE:
			case SDLK_RETURN:
				startLevel(0);
			break;
			
			default: {}
		}
	}
	
	void update() {
		for (int i = 0; i < 4; i++) {
			buttons[i]->update();
		}
	};

	void draw() const {
		ImageMap::get("background")->draw(0, 0);
		ImageMap::get("title")->draw(0, 0);
		
		for (int i = 0; i < 4; i++) {
			buttons[i]->draw();
		}
	};
	
	private:
	
	static void playAction (Button* b) {
		startLevel(0);
	}
	
	static void editorAction (Button* b) {
		transitionTo(editor);
	}
	
	static void aboutAction (Button* b) {
		if (aboutScreen == 0) {
			aboutScreen = new AboutScreen();
		}
		transitionTo(aboutScreen);
	}
	
	static void quitAction (Button* b) {
		exit(0);
	}
	
	Button* buttons[4];
};

#endif
