#ifndef _BUTTON_H_
#define _BUTTON_H_

#include "image.h"
#include "input.h"

class Button {
	public:
	
	typedef void (*callback_type)(Button*);
	
	Button (Image* image, int x, int y, callback_type callback)
		: image(image), x1(x), y1(y), x2(x + image->getWidth()), y2(y + image->getHeight()), callback(callback) {}
	
	void update () {
		if (isMouseOver() && Input::isMousePressed(1)) {
			callback(this);
		}
	}
	
	void draw () const {
		int frame = isMouseOver() ? 1 : 0;
		image->drawFrame(frame, x1, y1);
	}
	
	bool isMouseOver () const {
		int mx = Input::getMouseX();
		int my = Input::getMouseY();
		
		return (mx >= x1 && mx < x2 && my >= y1 && my < y2);
	}
	
	private:
	
	const Image* image;
	const int x1;
	const int y1;
	const int x2;
	const int y2;
	callback_type callback;
};

#endif

