#ifndef _SCREEN_H_
#define _SCREEN_H_

union SDL_Event;

class Screen {
	public:
	
	Screen () {}
	virtual ~Screen () {}
	
	/** Called when the screen is made active (could be multiple times). */
	virtual void init () {}
	
	/** Input has been received. */
	virtual void handleInput (SDL_Event& e) {}
	
	/** Called once per frame while the screen is active. */
	virtual void update () = 0;
	
	/** Called once per frame while the screen is active. */
	virtual void draw () const = 0;
	
	/** Called when the screen is replaced (could be multiple times). */
	virtual void finalise () {}
	
	/** Returns true if after the screen has been replaced it should be deleted. */
	virtual bool deletable () const { return false; }
	
	/** Shorthand for ScreenManager::getScene. */
	inline static Screen* getScreen ();
	
	/** Shorthand for ScreenManager::setScene. */
	inline static void setScreen (Screen* s);
};

class ScreenManager {
	public: 
	
	/** Get the active screen. */
	static Screen* getScreen () { return activeScreen; }
	
	/** Set the active screen. */
	static void setScreen (Screen* s);
	
	/** Send input to the active screen, checking for null-ness. */
	static void handleInput (SDL_Event& e) {
		if (activeScreen) {
			activeScreen->handleInput(e);
		}
	}
	
	/** Updates the active screen, checking for null-ness. */
	static void update () {
		if (activeScreen) {
			activeScreen->update();
		}
	}
	
	/** Updates the active screen, checking for null-ness. */
	static void draw () {
		if (activeScreen) {
			activeScreen->draw();
		}
	}
	
	private:
	
	static Screen* activeScreen;
};

// These have to be defined down here because of circular dependencies.
// They are inline, so cannot be placed in a cpp file.

Screen* Screen::getScreen () { return ScreenManager::getScreen(); }

void Screen::setScreen (Screen* s) { ScreenManager::setScreen(s); }

#endif

